%========================================
%========================================
min_all :-
	wdict(A),
	forall( member(W,A), wshow(W,2)).


%========================================
%========================================
subset(A,B) :-
	forall( member(E,A),
		member(E,B)).
%========================================
%
% Multiple Value Inheritance must be on
%  for this to accumulate values.
%========================================
lookupall(A,F,Vs) :-
	findall(Val,lookup(A,F,Val),Vals),
	meta_splice(Vals,Vs).


%================================================
%  LIST PROCESSING UTILITIES
%================================================

%==========================================
% Replace OLD with NEW in L1 to produce L2
%==========================================
replace_elem(_, _, [], _) :- !, fail.
replace_elem(Old, New, [Old|Z], [New|Z]) :- !.
replace_elem(Old, New, [X|Z], [X|Z2]) :-
  replace_elem(Old, New, Z, Z2).

%==========================================
%==========================================
flatten([], []).

flatten( [X|T], [X|T2] ) :-
  var(X),
  !, flatten(T, T2).

flatten([ [] | T], T2) :-
  !, flatten(T, T2).

flatten([ [H|T] | T2], [H1|T3]) :-
  flatten([H|T], [H1|T1]),
  !, flatten([T1|T2], T3).

flatten([H|T], [H|T2]) :-
  flatten(T, T2).
	
%==========================================
% Meta_Splice
%
% Splices together all elements
% of the given list where given
% list may be composed of atoms
% and/or arbitrarily nested lists.
% NOTE: ONLY DOES ONE level of flattening.
%
%==========================================
meta_splice(Lists,Result) :-
	meta_splice(Lists,[],Result1),
	flatten(Result1,Result).		%Changed May 19 1999

meta_splice([],Result,Result) :- !.

meta_splice([Only],Accum,Result) :- 
	!,
	splice(Accum,Only,Result).

meta_splice([First | [Second | Rest]],Accum,Result) :-
	splice(First,Second,S),
	append(Accum,S,Accum1),
	meta_splice(Rest,Accum1,Result).

%================================
% Splice
%================================
%
%P1 and P2 are Lists.
%
splice(P1,P2,P3) :-
	list(P1),
	list(P2),!,
	append(P1,P2,P3).

%-------------------
%P1 is a list
%P2 is not a list
%-------------------
splice(P1,P2,P3) :-
	list(P1),!,
	append(P1,[P2],P3).

%-------------------
%P1 is Not a List.
%P2 is the Empty List.
%-------------------
splice(P1,[],[P1]) :- !.

%-------------------
%P1 is not a list.
%P2 is a list and not empty
%-------------------
splice(P1,P2,[P1 | P2]) :-
	list(P2), !.

did_not_sign(Member,Document) :-
	not(isa_fact(signed(Member,Document,yes))).

%-------------------
%P1 and P2 are Atoms.
%-------------------
splice(P1,P2,[P1,P2]).


%====================================================
%  INPUT OUTPUT UTILITIES
%====================================================

pp_sentence(List) :-
	List = [First | URest],
	capitalize(First,CFirst),
	reverse([CFirst | URest],RList),
	RList = [Last | Rest],
	append_period(Last,PLast),
	NewList = [PLast | Rest],
	reverse(NewList,Final),
	writeseqnl(Final).


pp_openning_phrase(List) :-
	List = [First | URest],
	capitalize(First,CFirst),
	writeseq([CFirst | URest]).


writenl(A) :-
	write(A),
	nl.

writeseqnl(List) :-
	writeseq(List),
	nl.

writeseq([]) :- !.

writeseq([A | Rest]) :-
	write(A),
	write(' '),
	writeseq(Rest).


%=====================================================
%  RANDOM NUMBER and SELECTION UTILITIES
%=====================================================

%==============================================
%
% Randomly selects at most N items from
% the FromList and puts results in the ToList
%
%==============================================
randomly_select_at_most(N,[],[]) :- !.

randomly_select_at_most(N,FromList,ToList) :-
	length(FromList,L),
	(N =< L -> N1 is N ; N1 is L),
	get_randoms(N1,1,L,Rs),
	select_items(Rs,FromList,ToList).


%==============================================
%==============================================
select_items(SelectIndex,From,To) :- 
	select_items1(SelectIndex,From,[],To).

select_items1([],From,To,To) :- !.
select_items1([Index | Rest],From,ACCUM,To) :-
	mem(From,[Index],Element),
	select_items1(Rest,From,[Element | ACCUM],To).

%==============================================
% Generate Number of unique random numbers
%   between Begin and End and return in Randoms.
%
% Better way to do this is GENERATE Range
% then randomly select and delete from.
%
%
%==============================================
get_randoms(Number,Begin,End,Randoms) :-
	(Number =< End - Begin
	-> 	N is Number
	;	N is (End - Begin) + 1),
	get_randoms1(N,Begin,End,[],Randoms).

%-----------------------------------------------
% Randomly get N numbers between Begin and End
% that are not in the First List.
%-----------------------------------------------
get_randoms1(0,Begin,End,Randoms,Randoms) :- !.

get_randoms1(N,Begin,End,ACCUM,Randoms) :-
	unique_random(Begin,End,R,ACCUM),
	N1 is N - 1,
	get_randoms1(N1,Begin,End,[R | ACCUM],Randoms).


%============================================= 
% UNIQUE_RANDOM
% 	Chooses a random number in a given
%	range that is not already in the 
%	given list.
% 
%  Range_test ensures that some element in range is not
%  in unique list. This takes one
%  at most one full pass through range.
%
%  NOTE: Since we are scanning range, might
%  as well build list of numbers not in unique_list
%  and then randomly select from that list. 
%  Would in general have less misses.
%==============================================
unique_random(Begin,End,Random,UniqueList) :-
	L is End - Begin,
	range_test(Begin,End,UniqueList),
	repeat,
	random(Begin,End,Random),
	not(member(Random,UniqueList)),!.

%=============================================
% True if there is some element between
% Begin and End that is not in UniqueList
%=============================================

%Anything is unique to the empty set.
%
range_test(Begin,End,[]) :- !.

range_test(Begin,End,UniqueList) :-
	Begin < End,!,
	range_test(Begin,End,Begin,UniqueList).

range_test(Begin,End,UniqueList) :-
	Begin = End,
	member(Begin,UniqueList),!,fail.

range_test(Begin,End,UniqueList).

%--------------------------------------------
%--------------------------------------------
range_test(Begin,End,End,UniqueList) :- !,fail.

range_test(Begin,End,Current,UniqueList) :-
	Current < End,
	member(Current,UniqueList),!,
	C is Current + 1,
	range_test(Begin,End,C,UniqueList).

range_test(Begin,End,Current,UniqueList).


%============================================
%============================================	
random(From,To,R) :-
   To > From,!,
   Diff is (To - From) + 1,
   X is rand(Diff),
   R is ip(X) + From.


random(From,To,R) :-
   To = From,
   Diff is (To - From) + 1,
   X is rand(Diff),
   R is ip(X) + From.

